@@ -4,7 +4,7 @@
 from magicgui import magicgui
 from magicgui.widgets import Container
 from enum import Enum
-from napari_generic_simulator.baseSIMulator import import_cp
+from napari_generic_simulator.baseSIMulator import import_cp, import_torch, torch_GPU
 from napari_generic_simulator.hexSIMulator import HexSim_simulator, RightHexSim_simulator
 from napari_generic_simulator.conSIMulator import ConSim_simulator
 from qtpy.QtWidgets import QWidget, QVBoxLayout, QLineEdit
@@ -24,7 +24,10 @@ class Accel(Enum):
     USE_NUMPY = 0
     if import_cp:
         USE_CUPY = 1
-    # USE_TORCH = 3
+    if import_torch:
+        USE_TORCH_CPU = 2
+        if torch_GPU:
+            USE_TORCH_GPU = 3
 
 class SIMulator(QWidget):
     """
@@ -54,25 +57,26 @@ class SIMulator(QWidget):
         self._viewer.layers.events.removed.connect(function.reset_choices)
         _layout.addWidget(function.native)
 
-    def parameters(self, SIM_mode=Sim_mode.HEXSIM_RIGHT_ANGLES, Polarisation=Pol.IN_PLANE, Acceleration=list(Accel)[-1],
-                   N: int = 512, pixel_size: float = 5.5, magnification: int = 60, NA: float = 1.1, n: float = 1.33,
-                   wavelength: float = 0.52, npoints: int = 500, zrange: float = 7.0, dz: float = 0.35,
-                   fwhmz: float = 3.0):
+    def parameters(self, SIM_mode=Sim_mode.HEXSIM_RIGHT_ANGLES, Polarisation=Pol.AXIAL, Acceleration=list(Accel)[-1],
+                   N: int = 512, pixel_size_μm: float = 5.5, magnification: int = 60, NA: float = 1.1, n: float = 1.33,
+                   wavelength_μm: float = 0.52, npoints: int = 500, zrange_μm: float = 7.0, dz_μm: float = 0.35,
+                   fwhmz_μm: float = 3.0, random_seed: int = 123):
         self.SIM_mode = SIM_mode.value
         self.Polarisation = Polarisation.value
         self.Acceleration = Acceleration.value
         self.N = N
-        self.pixel_size = pixel_size
+        self.pixel_size = pixel_size_μm
         self.magnification = magnification
         self.NA = NA
         self.n = n
-        self.wavelength = wavelength
+        self.wavelength = wavelength_μm
         self.npoints = npoints
-        self.zrange = zrange
-        self.dz = dz
-        self.fwhmz = fwhmz
+        self.zrange = zrange_μm
+        self.dz = dz_μm
+        self.fwhmz = fwhmz_μm
+        self.random_seed = random_seed
         self.par_list =[self.SIM_mode, self.Polarisation, self.Acceleration, self.N, self.pixel_size, self.magnification, self.NA, self.n, self.wavelength,
-                self.npoints, self.zrange, self.dz, self.fwhmz]
+                self.npoints, self.zrange, self.dz, self.fwhmz, self.random_seed]
 
     def set_att(self):
         """Sets attributes in the simulation class. Executed frequently to update the parameters"""
@@ -84,19 +88,23 @@ class SIMulator(QWidget):
             self.sim = ConSim_simulator()
 
         if self.Polarisation == Pol.IN_PLANE.value:
-            self.sim.pol = 'in-plane'
+            self.sim.pol = 0
         elif self.Polarisation == Pol.AXIAL.value:
-            self.sim.pol = 'axial'
+            self.sim.pol = 1
         elif self.Polarisation == Pol.CIRCULAR.value:
-            self.sim.circular = 'circular'
+            self.sim.circular = 2
 
         if self.Acceleration == Accel.USE_NUMPY.value:
-            self.sim.use_cupy = False
-        elif Accel.USE_CUPY:
-            self.sim.use_cupy = True
-            if not import_cp:
-                print('No cupy present')
-                self.sim.use_cupy = False
+            self.sim.acc = 0
+        if hasattr(Accel, 'USE_CUPY'):
+            if self.Acceleration == Accel.USE_CUPY.value:
+                self.sim.acc = 1
+        if hasattr(Accel, 'USE_TORCH_CPU'):
+            if self.Acceleration == Accel.USE_TORCH_CPU.value:
+                self.sim.acc = 2
+        if hasattr(Accel, 'USE_TORCH_GPU'):
+            if self.Acceleration == Accel.USE_TORCH_GPU.value:
+                self.sim.acc = 3
 
         self.sim.N = self.N
         self.sim.pixel_size = self.pixel_size
@@ -108,6 +116,7 @@ class SIMulator(QWidget):
         self.sim.zrange = self.zrange
         self.sim.dz = self.dz
         self.sim.fwhmz = self.fwhmz
+        self.sim.random_seed = self.random_seed
 
     def start_simulator(self):
         """
@@ -141,10 +150,9 @@ class SIMulator(QWidget):
             print(e)
         self.used_par_list = [self.SIM_mode, self.Polarisation, self.Acceleration, self.N, self.pixel_size,
                               self.magnification, self.NA, self.n, self.wavelength, self.npoints, self.zrange, self.dz,
-                              self.fwhmz]
+                              self.fwhmz, self.random_seed]
         return self
 
-
     def show_raw_img_sum(self, show_raw_img_sum: bool=False):
         if show_raw_img_sum:
             if hasattr(self.sim, 'img_sum_z'):
@@ -161,7 +169,7 @@ class SIMulator(QWidget):
         if show_3D_psf:
             if hasattr(self.sim, 'psf_z0'):
                 if self.used_par_list != self.par_list:
-                    self.messageBox.setText('Parameters changed! Calculate the raw-image stack first!')
+                    self.messageBox.setText('Parameters changed!')
                 else:
                     try:
                         self._viewer.add_image(self.sim.psf_z0, name='PSF in x-y plane')
